package edu.fudan.cube.core;

import static java.awt.Color.BLUE;
import static java.awt.Color.GRAY;
import static java.awt.Color.GREEN;
import static java.awt.Color.PINK;
import static java.awt.Color.RED;
import static java.awt.Color.YELLOW;

import java.awt.Color;
import java.awt.image.BufferedImage;
import java.util.Arrays;

/**
 * Cube is a type of {@link GraphComposite}. Contains 8 points and can get the
 * edges and sides from it.
 * 
 * @author Lei Huang
 */
public class Cube implements GraphComposite {
    private Point[] points;
    private Color[] sideColor;
    
	/*
	 * default side colors
	 */
    private static final Color[] SIDE_COLOR = new Color[]{
        RED,
        GREEN,
        GRAY,
        BLUE,
        YELLOW,
        PINK
    };
    
    Cube() {
        points = new Point[8];
        sideColor = SIDE_COLOR;
    }

    Cube(Point[] points) {
        this.points = points;
        sideColor = SIDE_COLOR;
    }
    
    public void setSideColor(int index, Color color) {
        this.sideColor[index] = color;
    }
    
    public Color[] getSideColor() {
        return sideColor;
    }
    
    public Point[] getPoints() {
        return points;
    }
    
	/**
	 * Create an unit cube for use.
	 */
    public static Cube createCube() {
        Cube cube = new Cube();
        cube.points[0] = new Point(-1, -1, -1);
        cube.points[1] = new Point(-1, -1, 1);
        cube.points[2] = new Point(-1, 1, -1);
        cube.points[3] = new Point(-1, 1, 1);
        cube.points[4] = new Point(1, -1, -1);
        cube.points[5] = new Point(1, -1, 1);
        cube.points[6] = new Point(1, 1, -1);
        cube.points[7] = new Point(1, 1, 1);
        return cube;
    }

	/**
	 * @return An array contains 12 edges of this cube.
	 */
    public Line[] getEdges() {
        Line[] edges = new Line[12];
        edges[0] = new Line(new Point[]{points[0], points[1]});
        edges[1] = new Line(new Point[]{points[2], points[3]});
        edges[2] = new Line(new Point[]{points[4], points[5]});
        edges[3] = new Line(new Point[]{points[6], points[7]});
        edges[4] = new Line(new Point[]{points[0], points[2]});
        edges[5] = new Line(new Point[]{points[1], points[3]});
        edges[6] = new Line(new Point[]{points[4], points[6]});
        edges[7] = new Line(new Point[]{points[5], points[7]});
        edges[8] = new Line(new Point[]{points[0], points[4]});
        edges[9] = new Line(new Point[]{points[1], points[5]});
        edges[10] = new Line(new Point[]{points[2], points[6]});
        edges[11] = new Line(new Point[]{points[3], points[7]});
        return edges;
    }
    
	/**
	 * @return An array contains 8 sides of this cube.
	 */
    public Quad[] getSides() {
        Quad[] sides = new Quad[6];
        sides[0] = new Quad(new Point[]{points[0], points[1], points[3], points[2]});
        sides[1] = new Quad(new Point[]{points[4], points[5], points[7], points[6]});
        sides[2] = new Quad(new Point[]{points[0], points[1], points[5], points[4]});
        sides[3] = new Quad(new Point[]{points[2], points[3], points[7], points[6]});
        sides[4] = new Quad(new Point[]{points[0], points[2], points[6], points[4]});
        sides[5] = new Quad(new Point[]{points[1], points[3], points[7], points[5]});
        for(int index = 0; index < 6; index++) {
            sides[index].setColor(sideColor[index]);
        }
        return sides;
    }

	/**
	 * @return Cube rotate with axis (-y, x). Rotate the points of the cube
	 *         respectively.
	 */
    public Cube rotate(double x, double y) {
        Point points[] = new Point[8];
        for(int index = 0; index < 8; index++) {
            points[index] = getPoints()[index].rotate(x, y);
        }
        return new Cube(points);
    }

	/**
	 * Draw the cube, the parameter color is unused.
	 */
    @Override
    public void draw(BufferedImage image, Color color) {
        Quad[] sides = getSides();
        Arrays.sort(sides);
        for(Quad side : sides) {
            side.draw(image, side.getColor());
        }
    }
}
