package edu.fudan.cube.core;

import java.awt.Color;
import java.awt.image.BufferedImage;
import java.util.Arrays;

import edu.fudan.cube.DrawUtil3D;

/**
 * A quad is a side of the cube. Can be sort by the z-index of the points.
 * 
 * @author Lei Huang
 */
public class Quad implements GraphComposite, Comparable<Quad> {
    Point[] points;
    Color color;

    public Quad(Point[] points) {
        this.points = points;
    }
    
    public void setColor(Color color) {
        this.color = color;
    }
    
    public Color getColor() {
        return color;
    }
    
    public Point[] getPoints() {
        return new Point[]{points[0], points[1], points[2], points[3]};
    }
    
	/**
	 * @return Four edges of the side
	 */
    public Line[] getEdges() {
        Line[] edges = new Line[4];
        edges[0] = new Line(new Point[]{points[0], points[1]});
        edges[1] = new Line(new Point[]{points[2], points[3]});
        edges[2] = new Line(new Point[]{points[0], points[3]});
        edges[3] = new Line(new Point[]{points[1], points[2]});
        return edges;
    }

    @Override
    public void draw(BufferedImage image, Color color) {
		// if color is not null, draw this side
        if(color != null) drawContent(image, color);
		// draw four edges of this side
        drawLines(image, Color.BLACK);
    }

    private void drawContent(BufferedImage image, Color color) {
        Point[] quadPoint = new Point[4];

		// get the rect contains the side
        int minX = 400, maxX = 0, minY = 400, maxY = 0;
        for(int index = 0; index < 4; index++) {
            quadPoint[index] = DrawUtil3D.project(points[index]);
            minX = Math.min(minX, (int) Math.round(quadPoint[index].getX()));
            maxX = Math.max(maxX, (int) Math.round(quadPoint[index].getX()));
            minY = Math.min(minY, (int) Math.round(quadPoint[index].getY()));
            maxY = Math.max(maxY, (int) Math.round(quadPoint[index].getY()));
        }

		// check every pixel can draw the pixel inside.
        for(int x = minX; x <= maxX; x++) {
            for(int y = minY; y <= maxY; y++) {
                if(isInside(x, y, quadPoint)) {
                    DrawUtil3D.drawPixel(image, x, y, color);
                }
            }
        }
    }

	/*
	 * use look around method to check if a point is inside a convex polygon.
	 */
    private boolean isInside(int x, int y, Point[] points) {
        double total = 0;
        for(int index = 0; index < 4; index++) {
            double angle = Math.abs(Math.atan2(y - points[index].getY(), x - points[index].getX()) -
                    Math.atan2(y - points[(index+1)%4].getY(), x - points[(index+1)%4].getX()));
            while(angle >= 2 * Math.PI) angle -= 2 * Math.PI;
            if(angle >= Math.PI) angle = 2 * Math.PI - angle;
            total += angle;
        }
        return Math.abs(total - 2 * Math.PI) < 1e-5;
    }

	/*
	 * draw four edges of this side.
	 */
    private void drawLines(BufferedImage image, Color color) {
        for(Line edge : getEdges()) {
            edge.draw(image, color);
        }
    }

    public double getMinZ() {
        double minZ = 1e100;
        for(Point point : getPoints()) {
            minZ = Math.min(minZ, point.getZ());
        }
        return minZ;
    }

    @Override
    public int compareTo(Quad quad) {
        Point[] points = getPoints();
        Point[] otherPoints = quad.getPoints();
        Arrays.sort(points);
        Arrays.sort(otherPoints);

        for(int index=0; index<4; index++) {
            int delta = points[index].compareTo(otherPoints[index]);
            if(delta != 0) return delta;
        }
        return 0;
    }

}
